# Generated by Django 3.2 on 2022-11-10 11:28

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('apis_metainfo', '0006_remove_text_lang'),
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='AmpelTemp',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', models.CharField(choices=[('red', 'red'), ('yellow', 'yellow'), ('green', 'green')], default='red', max_length=300)),
                ('note', models.TextField(blank=True, max_length=2000, null=True)),
                ('temp_entity', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='apis_metainfo.tempentityclass')),
            ],
        ),
        migrations.CreateModel(
            name='AmpelSettings',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('_active', models.BooleanField(default=False)),
                ('_content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype')),
            ],
        ),
        migrations.CreateModel(
            name='AmpelGeneric',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField()),
                ('status', models.CharField(choices=[('red', 'red'), ('yellow', 'yellow'), ('green', 'green')], default='red', max_length=300)),
                ('note', models.TextField(blank=True, max_length=2000, null=True)),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype')),
            ],
            options={
                'unique_together': {('object_id', 'content_type')},
            },
        ),
    ]



# Generated by Django 3.2 on 2022-11-10 11:50

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('apis_ampel', '0001_initial'),
    ]

    operations = [
        migrations.RenameField(
            model_name='ampelsettings',
            old_name='_active',
            new_name='active',
        ),
        migrations.RenameField(
            model_name='ampelsettings',
            old_name='_content_type',
            new_name='content_type',
        ),
    ]



AMPEL_SETTINGS = {
    'relations':[
        {'PersonPerson': False},
        {'PersonInstitution': False},
        {'PersonPlace': False},
        {'PersonWork': False},
        {'PersonEvent': False},
        {'InstitutionInstitution': False},
        {'InstitutionPlace': False},
        {'InstitutionWork': False},
        {'InstitutionEvent': False},
        {'PlacePlace': False},
        {'PlaceWork': False},
        {'PlaceEvent': False},
        {'WorkWork': False},
        {'EventWork': False},
        {'EventEvent': False},
    ],
    'entities':
    [ 
        {'Person': False},
        {'Institution': False},
        {'Place': False},
        {'Event': False},
        {'Work': False},
    ]
}


from .models import AmpelGeneric, AmpelTemp, AmpelSettings
from django.shortcuts import render
from django.http import HttpResponse, JsonResponse
from django.views.generic import View, TemplateView
from django.utils.decorators import method_decorator
# Create your views here.
from django.template.loader import render_to_string
from django.contrib.auth.decorators import login_required, user_passes_test
from .fallback_settings import AMPEL_SETTINGS
from django.contrib.contenttypes.models import ContentType
from .helper_functions import get_or_create_ampel


# TODO add check for authentication

class AmpelDocsView(View):
    def get(self, request):
        template = "./ampel/ampel_documentation.html"

        return render(request, template, {})

class AmpelSettingsView(View):

    # def setup(self, request, *args, **kwargs):
    #     ampel_id = kwargs.get('ampel_id', None)
    #     self.ampel_id = ampel_id
    #     print(self.ampel_id, "ampel id")
   
    #     return super().setup(request, *args, **kwargs)

    def get(self, request):
        template = "./ampel/ampel_settings_template.html"
        rels = AMPEL_SETTINGS["relations"]
        ents = AMPEL_SETTINGS["entities"]
        #AmpelSettings.objects.all().delete()
        for el in ents+rels:
            for k,v in el.items():
                ct = ContentType.objects.get(model=k)
                a, c = AmpelSettings.objects.get_or_create(content_type=ct)
                # if c:
                #     model_class = ct.model_class()
                #     for obj in model_class.objects.all():
                #         try: 
                #             ag = AmpelGeneric.objects.create(content_type=ct, content_object=obj)
                #         except Exception as e:
                #             print(e)
                #         print("created new ampel for ", ag)
                #     print("created: ", a, c)
                #     a.active = v
                #     a.save()
        models = AmpelSettings.objects.all()
        return render(request, template, {"models":models})

    def post(self, request, **kwargs):
        instance = None
        success = False
        msg = ""

        #ampel_id = request.POST.get("ampel_id")
        ampel_id = kwargs.get("ampel_id")
        try:
            if ampel_id:
                instance = AmpelSettings.objects.get(id=ampel_id)
                instance.active = not instance.active
                instance.save()
                success = True
        except Exception as e:
            msg = str(e)
        
        return JsonResponse({"success":success, "msg":msg, "status":instance.active})


class FetchAmpelStatusView(View):
    def get(self, request):
        success = True
        msg = "success"
        ampel_pk = request.POST.get("ampel_pk")
        #status = request.POST.get("status")
        try:
            ampel = get_or_create_ampel(ampel_pk)
            ampel.status = status
            ampel.save()
        except Exception as e:
            success = False
            msg = f"something went wrong\n{e}"
        

        return JsonResponse({"success":success, "msg":msg, "status":None})


class UpdateAmpelStatusView(View):
    def post(self, request):
        success = True
        msg = "success"
        ampel_pk = request.POST.get("id")
        status = request.POST.get("status")
        try:
            ampel = get_or_create_ampel(ampel_pk, status)
            ampel.status = status
            ampel.save()
        except Exception as e:
            success = False
            msg = f"something went wrong\n{e}"
        

        return JsonResponse({"success":success, "msg":msg})


class HandleAmpelNoteView(View):

    def get(self, request, **kwargs):
        success = True
        msg = "success"
        note = ""
        try:
            ampel_pk = kwargs.get("ampel_pk")
            ampel = get_or_create_ampel(ampel_pk)
            note = ampel.note
            if not note:
                note = ""
        except Exception as e:
            success = False
            msg = f"{e}"

        return JsonResponse({"success":success, "msg":msg, "text":note})

    def post(self, request):
        print("post note called")
        success = True
        msg = "success"
        ampel_pk = request.POST.get("id")
        try:
            ampel = get_or_create_ampel(ampel_pk)
            ampel.note = request.POST.get("note_text")
            ampel.save()
        except Exception as e:
            success = False
            msg = f"{e}"
        
        return JsonResponse({"success":success, "msg":msg})




from django.db import models
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.conf import settings
from apis_core.apis_entities.models import TempEntityClass

class AmpelGeneric():
    pass
# class AmpelGeneric(models.Model):
#     ampel_choices = [
#         ("red", "red"),
#         ("yellow", "yellow"),
#         ("green", "green"),
    
#     ]

#     class Meta:
#         unique_together = [["object_id", "content_type"]]

#     content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
#     object_id = models.PositiveIntegerField()
#     content_object = GenericForeignKey('content_type', 'object_id')
    
#     status = models.CharField(max_length=300, choices=ampel_choices, default="red")
#     note = models.TextField(max_length=2000, null=True, blank=True)

#     def __str__(self):
#         return f"Ampel [{self.status}] - {str(self.content_object)}"

class AmpelTemp(models.Model):
    #TODO: rename to Ampel
    ampel_choices = [
    ("red", "red"),
    ("yellow", "yellow"),
    ("green", "green"),

    ]

    temp_entity = models.OneToOneField(TempEntityClass, on_delete=models.CASCADE, related_name="ampel", primary_key=True)
    status = models.CharField(max_length=300, choices=ampel_choices, default="red")
    note = models.TextField(max_length=2000, null=True, blank=True)

    def __str__(self):
        return f"Ampel [{self.status}] - {str(self.temp_entity)}"


class AmpelSettings(models.Model):
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    active = models.BooleanField(null=False, blank=False, default=False)

    
        



from django.urls import path
from django.conf.urls import url
#from .views import HandleAmpelNoteView, UpdateAmpelStatusView
from .views import AmpelSettingsView, FetchAmpelStatusView, UpdateAmpelStatusView, HandleAmpelNoteView, AmpelDocsView

app_name = "apis_ampel"

urlpatterns = [
    url(r"ampel_page/", AmpelSettingsView.as_view(), name="ampel_settings"),
    url(r"ampel_docs/", AmpelDocsView.as_view(), name="ampel_docs"),
    url(r"ampel_settings/update/(?P<ampel_id>[0-9]+)/$", AmpelSettingsView.as_view(), name="update_ampel_view"),
   # url(r"ampel_entity/fetch_ampel_status", FetchAmpelStatusView.as_view(), name="fetch_ampel_status"),

    #url(r"entity/handle_note/get/(?P<ampel_pk>[0-9]+)/$", HandleAmpelNoteView.as_view(), name="get_note_text"),
    url(r"ampel_entity/handle_note/$", HandleAmpelNoteView.as_view(), name="update_note_text"),
    url(r"ampel_entity/update_ampel_status", UpdateAmpelStatusView.as_view(), name="update_ampel_instance"),
]


from django.apps import AppConfig


class ApisAmpelConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'apis_ampel'


    #could overwrite ready method here to run code when app is fully loaded



from .models import AmpelTemp, AmpelSettings
from apis_core.apis_entities.models import TempEntityClass
from django.conf import settings
from django.contrib.contenttypes.models import ContentType

# def get_ampel_color(model_class, instance_pk):
#     print("get_ampel_color called", model_class, instance_pk)
#     ct = ContentType.objects.get(model=model_class)
#     model = ct.model_class()
#     instance = model.objects.get(id=instance_pk)
#     test = AmpelGeneric.objects.filter(pk=instance_pk)
#     try:
#         if test and len(test) < 2:
#             color = test[0].status
#         else: 
#             new_instance = AmpelGeneric.objects.create(content_type=ct, content_object=instance)
#             new_instance.save()
#             color = new_instance.status
#     except Exception as e:
#         print("SOMETHING WENT WRONG", model_class, instance_pk, e)
#     return color


def get_or_create_ampel(pk, status=None):
    """
    Enables the lazy handling of the ampel: Auto-creates the ampel for a given entity or relation when it is being accessed for the first time or returns the exisiting instance.

    pk: TempEntityClass-instance primary key: Note that all entitites and relations share their primary key with the TempEntityClass they inherited from. 
        So all keys are continous and singular. Thefore, no check for the specific entity- or relation-type is needed.
    status: optional param; give an initial state or change the existing state. Expects a value of "red", "yellow" or "green".
    return: an Ampel-instance
    """
    a = AmpelTemp.objects.filter(pk=pk)

    if not status:
        if a:
            return AmpelTemp.objects.get(pk=pk)
        else: 
            a = AmpelTemp.objects.create(temp_entity=TempEntityClass.objects.get(pk=pk))
            a.save()
            return a 
    else:
        if a:
            a = AmpelTemp.objects.get(pk=pk)
            a.status = status
            a.save()
            return a
        else: 
            a = AmpelTemp.objects.create(temp_entity=TempEntityClass.objects.get(pk=pk), status=status)
            a.save()
            return a 

def is_ampel_active(model_name:str):
    """
    Checks if the display of the Ampel is enabled in the AmpelSettings for a given model. 
    model_name: name of the model to check for; like Person, Institution or PersonInstitution, etc.

    return: boolean value representing if model is active in AmpelSettings.
    
    """
    print(f"In ampel active, model name is: {model_name}")
    try: 
        as_instance = AmpelSettings.objects.get(content_type=ContentType.objects.get(model=model_name))
        return as_instance.active
    except Exception as e:
        print(e)
        return False
    

def get_ampel_color(*args, **kwargs):
    # DELETE
    pass





# Generated by Django 3.2 on 2022-11-11 10:22

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('apis_metainfo', '0006_remove_text_lang'),
        ('apis_ampel', '0004_alter_ampeltemp_temp_entity'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='ampeltemp',
            name='id',
        ),
        migrations.AlterField(
            model_name='ampeltemp',
            name='temp_entity',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, primary_key=True, related_name='ampel', serialize=False, to='apis_metainfo.tempentityclass'),
        ),
    ]



# Generated by Django 3.2 on 2022-11-11 09:35

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('apis_ampel', '0002_auto_20221110_1150'),
    ]

    operations = [
        migrations.DeleteModel(
            name='AmpelGeneric',
        ),
    ]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['apis_ampel', 'apis_ampel.migrations']

package_data = \
{'': ['*'], 'apis_ampel': ['templates/ampel/*']}

install_requires = \
['Django==3.2.0',
 'PyYAML>=5.3.1,<6.0.0',
 'SPARQLWrapper>=1.8.5,<2.0.0',
 'acdh-django-charts>=0.5.3,<0.6.0',
 'convertdate>=2.3.0,<3.0.0',
 'dj-database-url>=0.5.0,<0.6.0',
 'django-admin-csvexport>=1.9,<2.0',
 'django-allow-cidr>=0.3.1,<0.4.0',
 'django-autocomplete-light>=3.9.4,<4.0.0',
 'django-cors-headers>=3.13.0,<4.0.0',
 'django-crispy-forms>=1.14.0,<2.0.0',
 'django-crum>=0.7.9,<0.8.0',
 'django-csp>=3.7,<4.0',
 'django-extensions>=3.1.3,<4.0.0',
 'django-filter>=22.1,<23.0',
 'django-guardian>=2.3.0,<3.0.0',
 'django-leaflet>=0.27.1,<0.28.0',
 'django-model-utils>=4.1.1,<5.0.0',
 'django-reversion==3.0.8',
 'django-summernote>=0.8.11,<0.9.0',
 'django-tables2>=2.4.1,<3.0.0',
 'djangorestframework-csv>=2.1.0,<3.0.0',
 'djangorestframework-jsonschema>=0.1.1,<0.2.0',
 'djangorestframework-xml>=2.0.0,<3.0.0',
 'djangorestframework==3.12.2',
 'drf-spectacular>=0.22.0,<0.23.0',
 'gunicorn>=20.0.4,<21.0.0',
 'jmespath>=0.10.0,<0.11.0',
 'jsonschema>=3.2.0,<4.0.0',
 'lxml>=4.6.2,<5.0.0',
 'mysqlclient>=2.0.3,<3.0.0',
 'numpy==1.22',
 'openpyxl>=3.0.10,<4.0.0',
 'pandas==1.1.5',
 'python-Levenshtein>=0.12.2,<0.13.0',
 'rdflib>=6.1.1,<6.2.0',
 'regex>=2020.11.13,<2021.0.0',
 'requests>=2.25.0,<3.0.0',
 'sentry-sdk>=1.5.12,<1.6.0',
 'tablib>=3.0.0,<4.0.0',
 'tqdm>=4.62.3,<5.0.0',
 'typesense>=0.14.0,<0.15.0',
 'unicodecsv>=0.14.1,<0.15.0',
 'whitenoise>=5.2.0,<6.0.0']

setup_kwargs = {
    'name': 'apis-ampel',
    'version': '0.1.0',
    'description': 'Django-App for the APIS plattform. Adds status indicators to all entities that are displayed as traffic lights in list, detail and edit views.',
    'long_description': None,
    'author': 'Gregor Pirgie',
    'author_email': 'gregor.pirgie@oeaw.ac.at',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<3.9',
}


setup(**setup_kwargs)



from django_tables2 import TemplateColumn
from .helper_functions import get_ampel_color
from django.template import Context, Template


class AmpelTemplateColumn(TemplateColumn):
    def render(self, record, table, value, bound_column, **kwargs):
        print(record, value)
        model = self.extra_context.get("model")
        value = get_ampel_color(model, record.pk)
        context = getattr(table, "context", Context())

        print(self.extra_context)
        return super(AmpelTemplateColumn, self).render(record, table, value, bound_column, **kwargs)        
